/*
 *            File: ServerCrociera.java
 *          Autore: Roberto FULIGNI
 * Ultima modifica: 07/12/2023
 *
 *     Descrizione: Applicazione "Crociera panoramica in battello"
 *                  (modulo server)
 */

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.net.Socket;
import java.net.UnknownHostException;

public class ClientCrociera {
    final static String SERVER_DEFAULT = "localhost";
    final static int PORTA_SERVER = 5080;

    public static void main(String[] args) {
        // Ricava il nome ( indirizzo IP) del server dal primo argomento della riga di comando
        // Se l'argomento non esiste, si usa il server di default (localhost)
        String nomeServer = args.length > 0 ? args[0] : SERVER_DEFAULT;

        String cognome = "Rossi", nome = "Mario";
        int eta = 21;

        try (var sck = new Socket(nomeServer, PORTA_SERVER))
        {
            String loc = sck.getLocalSocketAddress().toString();
            String rem = sck.getRemoteSocketAddress().toString();
            System.out.format("Client: %s%n", loc);
            System.out.format("Server: %s%n", rem);

            var ois = new ObjectInputStream(sck.getInputStream());
            var oos = new ObjectOutputStream(sck.getOutputStream());

            String buffer = (String) ois.readObject();
            System.out.format("Esito del collegamento: %s%n", buffer);
            if (buffer.equals("ACCETTATO")) {
                Richiesta r = new Richiesta(cognome, nome, eta);
                oos.writeObject(r);
                Biglietto b = (Biglietto) ois.readObject();
                System.out.format("Biglietto: %s%n", b.toString());
                System.out.println("In attesa di ricevere il segnale di partenza...");
                buffer = (String) ois.readObject();
                System.out.println(buffer);
            }
            System.out.println("Chiusura del collegamento...");
            // Chiusura automatica alla fine del blocco try with resources

        } catch (UnknownHostException e) {
            System.err.format("Nome di server non valido: %s%n", e.getMessage());
        } catch (IOException e) {
            System.err.format("Errore durante la comunicazione con la biglietteria: %s%n",
                    e.getMessage());
        } catch (ClassNotFoundException e) {
            System.out.println(String.format("Classe non riconosciuta: ", e.getMessage()));
        }
    }
}