import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class ServerCrociera {
    final static int PORTA_SERVER = 5080;
    final static int MAX_BIGLIETTI = 2;         // Numero di biglietti da vendere prima di iniziare la crociera
    static final double PREZZO_MINORENNI = 25.00;   // Euro
    static final double PREZZO_MAGGIORENNI = 32.50; // Euro

    static Lock lck = new ReentrantLock();                      // Accesso in mutua esclusione alle variabili condivise
    static int bigliettiVenduti = 0;                            // Variabile condivisa
    static List<ObjectOutputStream> clientiAttesa = new ArrayList<>();      // Variabile condivisa, contiene gli outputStream
                                                                            // collegati ai clienti in attesa di partire
                                                                            // e su cui saranno inviati i messaggi di partenza
    static ExecutorService pool = Executors.newFixedThreadPool(4);

    public static void main(String[] args) {
        try (ServerSocket server = new ServerSocket(PORTA_SERVER)) {
            System.out.format("Biglietteria in ascolto su: %s%n", server.getLocalSocketAddress());
            while (true) {
                Socket tempSck;
                try {
                    tempSck = server.accept();
                    pool.execute(() -> venditaBiglietto(tempSck));
                }
                catch (IOException e) {
                    if (!server.isClosed()) {
                        System.err.println(String.format("Errore nella gestione di nuove connessioni: %s", e.getMessage()));
                    }
                }
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private static void venditaBiglietto(Socket sck)  {
        // Elaborazione della richiesta di acquisto in mutua esclusione
        lck.lock();
        try {
            var oos = new ObjectOutputStream(sck.getOutputStream());
            var ois = new ObjectInputStream(sck.getInputStream());
            String msg;

            String saClient = sck.getRemoteSocketAddress().toString();
            System.out.format("Nuova connessione da %s: ", saClient);

            if (bigliettiVenduti == MAX_BIGLIETTI) {
                msg = "NON ACCETTATO";
                System.out.println(msg);
                oos.writeObject(msg);
                sck.close();
                return;
            }
            msg = "ACCETTATO";
            System.out.println(msg);
            oos.writeObject(msg);

            Richiesta r = null;
            r = (Richiesta) ois.readObject();

            double prezzo = r.getEta() < 18 ? PREZZO_MINORENNI : PREZZO_MAGGIORENNI;

            Biglietto b = new Biglietto(
                    "Arona",
                    r.getCognome(),
                    r.getNome(),
                    r.getEta(),
                    prezzo);

            oos.writeObject(b);
            System.out.format("\tBiglietto emesso: %s%n", b);    // Stampa a video il biglietto invocando
                                                                // automaticamente il metodo b.toString()

            clientiAttesa.add(oos);

            bigliettiVenduti++;

            // Se la vendita è stata completata, occorre notificare la partenza,
            // a tutti i clienti in possesso di biglietto

            if (bigliettiVenduti == MAX_BIGLIETTI) {
                for(var s : clientiAttesa) {
                    s.writeObject("PARTENZA");
                    s.close();
                }
                clientiAttesa.clear();
                System.out.println("Battello in partenza...");
            }

        } catch (IOException e) {
            System.err.println(String.format("Errore di socket: %s", e.getMessage()));
        } catch (ClassNotFoundException e) {
            System.out.println(String.format("Classe non riconosciuta: ", e.getMessage()));
        } finally {
            lck.unlock();
        }
    }
}